{$I DEFINES.INC}
unit ddfossil;
{$S-,V-,R-}

interface

uses
  EleNorm, VpSysLow;
  
var
 port_num: integer;

procedure async_send(c: char);
procedure async_send_string(s: string);
function async_receive(var ch: char): boolean;
function async_carrier_drop: boolean;
function async_carrier_present : boolean;
function async_buffer_check: boolean;
function async_init_fossil: boolean;
procedure async_deinit_fossil;
procedure async_flush_output;
procedure async_purge_output;
procedure async_purge_input;
procedure async_set_dtr(state: boolean);
procedure async_watchdog_on;
procedure async_watchdog_off;
procedure async_warm_reboot;
procedure async_cold_reboot;
procedure async_set_baud(n: longint);
procedure async_set_baudBnu(n: longint);
procedure async_set_flow(SoftTran,Hard,SoftRecv: boolean);
procedure Async_Buffer_Status(var Insize,Infree,OutSize,Outfree: word;
                              var fossilname:string);

implementation

var
  DidInit: boolean = false;
  LastChar: Char;

procedure async_send(c: char);
begin;
//writeln('async_send: ' + c);
  if Not(DidInit) then Exit;
  
  Com_SendChar(c);
end;

procedure async_send_string(s: string);
begin;
  if Not(DidInit) then Exit;
  
  Com_SendString(s);
end;

function async_receive(var ch: char): boolean;
begin;
//writeln('async_receive');
  ch := #0;
  async_receive := false;
  
  if Not(DidInit) then Exit;
  
  // Loop until we get a key we want
  while (Com_CharAvail) and (ch = #0) do
  begin
    // Get next char
    ch := Com_GetChar;
      
    // Handle translating the LF character
    if (ch = #10) then
    begin
      if (LastChar = #13) then
      begin
        // Last was CR, so ignore LF
        LastChar := #0;
        ch := #0;
      end else
      begin
        // Last was not CR, so translate this LF to CR
        ch := #13;
        LastChar := ch;
        async_receive := true;
        Exit;
      end;
    end else
    begin
      LastChar := ch;
      async_receive := true;
      Exit;
    end;
  end;
end;

function async_carrier_drop: boolean;
begin;
//writeln('async_carrier');
  async_carrier_drop := false;
  
  if Not(DidInit) then Exit;

  async_carrier_drop := NOT Com_Carrier;
end;

function async_carrier_present: boolean;
begin;
//writeln('async_Carrier_present');
  async_carrier_present := true;
  
  if Not(DidInit) then Exit;

  async_carrier_present := Com_Carrier;
end;

function async_buffer_check: boolean;
begin;
//writeln('async_buffer_check');
  async_buffer_check := false;
  
  if Not(DidInit) then Exit;
  
  async_buffer_check := Com_CharAvail;
end;

function async_init_fossil: boolean;
begin;
//writeln('async_init_fossil: ', port_num);

  if DidInit then Exit;

  DidInit := true;
  Com_StartUp(2);
  Com_SetDontClose(true);
  Com_OpenQuick(port_num + 1); // COMIO lessens port_num by 1
  async_init_fossil := true;
end;

procedure async_deinit_fossil;
begin;
//writeln('async_deinit_fossil');
  if Not(DidInit) then Exit;

  Com_ShutDown;
end;

procedure async_set_dtr(state: boolean);
begin;
  // Do Nothing
end;

procedure MySlice;
begin
  SysCtrlSleep(1);
end;

procedure async_flush_output;
begin;
//writeln('async_flush_output');

  if Not(DidInit) then Exit;

  Com_FlushOutBuffer(MySlice);
end;

procedure async_purge_output;
begin;
//writeln('async_purge_output');

  if Not(DidInit) then Exit;

  Com_PurgeOutBuffer;
end;

procedure async_purge_input;
begin;
//writeln('async_purge_input');

  if Not(DidInit) then Exit;

  Com_PurgeInBuffer;
end;

procedure async_watchdog_on;
begin;
  // Do Nothing
end;

procedure async_watchdog_off;
begin;
  // Do Nothing
end;

procedure async_warm_reboot;
begin;
  // Do Nothing
end;

procedure async_cold_reboot;
begin;
  // Do Nothing
end;

procedure async_set_baud(n: longint);
begin;
  // Do Nothing
end;

procedure async_set_baudBnu(n: longint);
begin;
  // Do Nothing
end;

procedure async_set_flow(SoftTran,Hard,SoftRecv: boolean);
begin;
  // Do Nothing
end;

procedure Async_Buffer_Status(var Insize,Infree,OutSize,Outfree: word;
                              var fossilname:string);
var FInFree, FOutFree, FInUsed, FOutUsed: Longint;
begin;
//writeln('async_buffer_status');
  if Not(DidInit) then
  begin
  
    Exit;
  end;

  Com_GetBufferStatus(FInFree, FOutFree, FInUsed, FOutUsed);
  Insize := FInFree + FInUsed;
  Infree := FInFree;
  Outsize := FOutFree + FOutUsed;
  Outfree := FOutFree;
end;

end.
